﻿//DESCRIPTION: Convert ID footnotes to stand-alone footnotes
// Peter Kahrel

try {
	footnotes_in_columns();
} catch (e) {
	alert (e.message + ' (line ' + e.line + ')')
};

//-----------------------------------------------------------------------------------------------

function footnotes_in_columns (doc) {
	var resetRuler = setRuler(); //We need rulers per page
	var doc = app.activeDocument;
	var footn, first_frame, frame, fn, i;
	var styles = fix_styles (doc);
	fix_returns (doc, styles);
	// columnNotes is used to link footnote stories to their parent stories
	var columnNotes = {};
	var stories = doc.stories.everyItem().getElements();
	var fnStart = app.documents[0].footnoteOptions.startAt
	for (var j = 0; j < stories.length; j++) {
		footn = stories[j].footnotes.everyItem().getElements();
		if (footn.length > 0) {
			first_frame = footn[0].storyOffset.parentTextFrames[0];
			frame = place_frame (first_frame, styles);
			for (i = 0; i < footn.length; i++) {
				footn[i].parent.insertionPoints[footn[i].storyOffset.index+1].contents = String (footn[i].index + fnStart)
				footn[i].texts[0].move (LocationOptions.after, frame.parentStory.insertionPoints[-1]);
				frame.parentStory.insertionPoints[-1].contents = '\r';
			}
			if (frame.parentStory.characters[-1].contents === '\r') {
				frame.parentStory.characters[-1].remove();
			}
			frame.parentStory.characters[-1].remove();
			columnNotes[stories[j].id] = frame.parentStory.id;
		}
	}
	delete_notemarkers (doc);
	doc.insertLabel ('FootnoteStoryLinks', columnNotes.toSource());
	doc.insertLabel ('FootnoteObjectStyle', styles.ostyle.name);
	resetRuler();
}


function fix_returns (fn, styles) {
	// Find all second and further paragraphs in footnotes
	// and apply the next style non-destructively
	setupGrep ('(?s)(?<=\\r).+', '', {PS: styles.pstyle, FN: true});
	var found = app.activeDocument.findGrep();
	for (var i = 0; i < found.length; i++) {
		found[i].applyParagraphStyle (styles.next_pstyle, false);
	}
}



function place_frame (frame, styles) {
	var page;
	var gb = frame.geometricBounds;
	frame.hasOwnProperty('parentPage') ? page = frame.parentPage : page = frame.parent;
	if (page.marginPreferences.columnCount == 1) {
		gb[0] = gb[2]-100;
	} else {
		var left; 
		page.side == PageSideOptions.rightHand ? left = page.marginPreferences.left : left = page.marginPreferences.right;
		var right = page.marginPreferences.columnsPositions.pop()+left;
		gb = [gb[2]-100, left, gb[2], right];
	}
	var NewF = page.textFrames.add ({
		geometricBounds: gb,
		name: 'footnotes in columns',
	});
	NewF.applyObjectStyle (styles.ostyle);
	return NewF;
}


function delete_notemarkers (scope) {
	app.findGrepPreferences = app.changeGrepPreferences = null;
	app.findChangeGrepOptions.includeFootnotes = true;
	app.findGrepPreferences.findWhat = '~F';
	scope.changeGrep ();
}


function fix_styles (doc) {
	// Add a new style for the footnotes if they use [Basic or [No . . .]
	if (doc.footnoteOptions.footnoteTextStyle.name[0] == '[') {
		if (doc.paragraphStyles.item ('footnote_text') == null) {
			doc.paragraphStyles.add ({
				name:'footnote_text', 
				basedOn: doc.paragraphStyles[1],
			});
		}
		doc.footnoteOptions.footnoteTextStyle = doc.paragraphStyles.item ('footnote_text');
	}

	with (doc.footnoteOptions.footnoteTextStyle) {
		bulletsAndNumberingListType = ListType.numberedList;
		numberingExpression = '^#' + doc.footnoteOptions.separatorText;
		spaceBefore = doc.footnoteOptions.spaceBetween;
		numberingStartAt = app.documents[0].footnoteOptions.startAt;
	}

	// Add a fn style for second and further paragraphs in a note
	var next_name = doc.footnoteOptions.footnoteTextStyle.name + '_nonum';
	if (!doc.paragraphStyles.item (next_name).isValid) {
		doc.paragraphStyles.add ({
			basedOn: doc.footnoteOptions.footnoteTextStyle,
			name: next_name, 
			bulletsAndNumberingListType: ListType.NO_LIST,
		});
	}
	var next_style = doc.paragraphStyles.item (next_name);
	
	doc.footnoteOptions.separatorText = '';

	// Add char. style for the note references if it's currently set to [None] or (Ignore)
	if (doc.footnoteOptions.footnoteMarkerStyle == doc.characterStyles[0] || doc.footnoteOptions.footnoteMarkerStyle == null) {
		if (doc.characterStyles.item ('footnote_reference') == null) {
			doc.characterStyles.add ({
				name:'footnote_reference',
			});
		}
		doc.footnoteOptions.footnoteMarkerStyle = doc.characterStyles.item ('footnote_reference');
	}

	if (!doc.colors.item ('footnote_reference').isValid) {
		with (doc.footnoteOptions.footnoteMarkerStyle) {
			underline = true;
			underlineWeight = '12pt';
			underlineTint = 40;
			underlineColor = doc.colors.add ({
				name:'footnote_reference', 
				colorValue: [0, 100, 0, 0],
			});
		}
	}
	

	var n = 'footnotes_columns';
	if (doc.objectStyles.item(n) == null) {
		with (doc.objectStyles.add({name: n})) {
			basedOn = doc.objectStyles[0];
			enableFill = enableStroke = enableStrokeAndCornerOptions = false;
			enableParagraphStyle = true;
				appliedParagraphStyle = doc.footnoteOptions.footnoteTextStyle;
			enableTextFrameGeneralOptions = true;
				textFramePreferences.verticalJustification = VerticalJustification.bottomAlign;
				// Try and guess how many columns are needed in the notes frame
				if (app.activeWindow.activePage.marginPreferences.columnCount > 1) {
					textFramePreferences.textColumnCount = 1;
				} else {
					textFramePreferences.textColumnCount = 2;
				}
			enableTextWrapAndOthers = true;
				try {/*CS4+*/textWrapPreferences.textWrapMode = TextWrapModes.nextColumnTextWrap;}
				catch (_){/*CS3*/textWrapPreferences.textWrapType = TextWrapTypes.nextColumnTextWrap;}
				textWrapPreferences.textWrapOffset = doc.footnoteOptions.spacer;
			}
		}
	
	return {
		pstyle: doc.footnoteOptions.footnoteTextStyle, 
		next_pstyle: next_style,
		cstyle: doc.footnoteOptions.footnoteMarkerStyle, 
		ostyle: doc.objectStyles.item(n),
	}
}


function setRuler () {
	var r = app.activeDocument.viewPreferences.rulerOrigin;
	app.activeDocument.viewPreferences.rulerOrigin = RulerOrigin.pageOrigin;
	return function () {
		app.activeDocument.viewPreferences.rulerOrigin = r;
	}
}


function setupGrep (find, replace, options) {
	app.findGrepPreferences = app.changeGrepPreferences = null;
	app.findGrepPreferences.findWhat = find;
	app.changeGrepPreferences.changeTo = replace;
	if (options == undefined) options = {};
	app.findGrepPreferences.appliedParagraphStyle = options.PS || null;
	app.findGrepPreferences.appliedCharacterStyle = options.CS || null;
	app.findChangeGrepOptions.properties ={
		includeFootnotes: options.FN || false,
		includeMasterPages: options.M || false,
		includeHiddenLayers: options.HL || false,
		includeLockedLayersForFind: options.LL || false,
		includeLockedStoriesForFind: options.LS || false
	}
}